import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.comp.helper.Bootstrap;
import com.sun.star.comp.helper.BootstrapException;
import com.sun.star.container.XNameAccess;
import com.sun.star.frame.FrameSearchFlag;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XDesktop;
import com.sun.star.lang.Locale;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.table.XCellRange;
import com.sun.star.text.PlaceholderType;
import com.sun.star.text.XDependentTextField;
import com.sun.star.text.XTextFieldsSupplier;
import com.sun.star.text.XTextTablesSupplier;
import com.sun.star.uno.Exception;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.util.MalformedNumberFormatException;
import com.sun.star.util.XNumberFormats;
import com.sun.star.util.XNumberFormatsSupplier;
import com.sun.star.util.XRefreshable;

public class CreateTemplate {

	@SuppressWarnings("unchecked")
	static<T> T cast(Class<T> c, Object o) {
		return (T) UnoRuntime.queryInterface(c, o);
	}
	
	private static int getNumberFormatKey(XNumberFormats numberFormats, String format, Locale locale) throws MalformedNumberFormatException {
		int key = numberFormats.queryKey(format, locale, true);
		if( key < 0 ) {
			key = numberFormats.addNew(format, locale);
		}
		return key;
	}
	
	public static void main(String[] args) throws BootstrapException, Exception {
		
		XComponentContext      context  = Bootstrap.bootstrap();
		XMultiComponentFactory factory  = context.getServiceManager();
		XDesktop               desktop  = cast(XDesktop.class, factory.createInstanceWithContext("com.sun.star.frame.Desktop", context));
		XComponentLoader       loader   = cast(XComponentLoader.class, desktop);

		PropertyValue templateParam = new PropertyValue();
		templateParam.Name  = "AsTemplate";
		templateParam.Value = Boolean.TRUE;
		
		XComponent document = loader.loadComponentFromURL("file:///D:/Java/eclipse3.1/workspace/docs/Verffentlichungen/iX/example-files/invoice-demo.odt", "_blank", FrameSearchFlag.ALL, new PropertyValue[] { templateParam });

		XMultiServiceFactory serviceFactory = cast(XMultiServiceFactory.class, document);
		
		XTextFieldsSupplier textFieldsSupplier = cast(XTextFieldsSupplier.class, document);
		XNameAccess         textFieldMasters   = textFieldsSupplier.getTextFieldMasters();
		
		// Empfnger-Felder durch Platzhalter ersetzen
		XPropertySet nameMaster = cast(XPropertySet.class, textFieldMasters.getByName("com.sun.star.text.FieldMaster.User.Name"));
		XDependentTextField[] fields = (XDependentTextField[]) nameMaster.getPropertyValue("DependentTextFields");
		for(XDependentTextField field : fields) {
			XPropertySet        jumpEditProps = cast(XPropertySet.class, serviceFactory.createInstance("com.sun.star.text.TextField.JumpEdit"));
			XDependentTextField jumpEditField = cast(XDependentTextField.class, jumpEditProps);
			jumpEditProps.setPropertyValue("PlaceHolder",     "Kundenname");
			jumpEditProps.setPropertyValue("Hint",            "Geben Sie hier den Kundennamen ein");
			jumpEditProps.setPropertyValue("PlaceHolderType", new Short(PlaceholderType.TEXT));
			jumpEditField.attach(field.getAnchor());
		}
		
		// Absender-Felder durch konkreten Text ersetzen
		XPropertySet senderMaster = cast(XPropertySet.class, textFieldMasters.getByName("com.sun.star.text.FieldMaster.User.Absender"));
		fields = (XDependentTextField[]) senderMaster.getPropertyValue("DependentTextFields");
		for(XDependentTextField field : fields) {
			field.getAnchor().setString("Meine Firma");
		}
		
		// Muster fr den Wert der Rechnungsnummer setzen und Felder aktualisieren
		XPropertySet invoiceNoMaster = cast(XPropertySet.class, textFieldMasters.getByName("com.sun.star.text.FieldMaster.User.Rechnungsnummer"));
		invoiceNoMaster.setPropertyValue("Content", "R-2005-00017");
		cast(XRefreshable.class, textFieldsSupplier.getTextFields()).refresh();
		
		// Texttabelle holen
		XTextTablesSupplier textTablesSupplier = cast(XTextTablesSupplier.class, document);
		XCellRange textTable          = cast(XCellRange.class, textTablesSupplier.getTextTables().getByName("RechnungsTabelle"));
		
		XNumberFormatsSupplier numberFormatsSupplier = cast(XNumberFormatsSupplier.class, document); 
		XNumberFormats         numberFormats         = numberFormatsSupplier.getNumberFormats();
		int key = getNumberFormatKey(numberFormats, "[$$-409]#,##0.00;-[$$-409]#,##0.00", new Locale("en", "US", ""));
		
		// fr Zellbereich E2:B4 das Zahlenformat setzen
		XPropertySet cellRangeProperties = cast(XPropertySet.class, textTable.getCellRangeByName("D2:B4"));
		cellRangeProperties.setPropertyValue("NumberFormat", new Integer(key));
		
//		// fr Zellbereich D2:D3 das Zahlenformat setzen
//		cellRangeProperties = cast(XPropertySet.class, textTableCellRange.getCellRangeByName("D2:D3"));
//		cellRangeProperties.setPropertyValue("NumberFormat", new Integer(key));
		
		System.exit(0);
	}

}
